<h1 align="center">
  Nesxt
</h1>
<p align="center">A Next.js Integration Module for NestJS</p>
<div align="center">
<a href="https://www.npmjs.com/package/nesxt"><img src="https://img.shields.io/npm/v/nesxt?style=flat-square" alt="NPM Version" /></a>
<a href="https://github.com/omauricf/nesxt/blob/master/LICENSE"><img src="https://img.shields.io/github/license/omauricf/nesxt.svg" alt="Package License" /></a>
<a href="https://www.npmjs.com/~nesxt"><img src="https://img.shields.io/npm/dm/nesxt.svg" alt="NPM Downloads" /></a>
</div>

## Overview

Nesxt is a integration module that overrides the default render method of NestJS by implementing Next.js under the hood, as an template engine, allowing you to render reactive interfaces on demand. Hence, every technique for using the MVVM (Model-View-ViewModel) pattern in React will be seamless integrated and available to Nest.

## Features

- A per-page basis routing system
- Pre-rendering, both static generation (SSG) and server-side rendering (SSR)
- Extendable API endpoints
- Static Files Serving
- Automatic Code Splitting and Lazy Loading
- Client-side routing with optimised prefetching
- Built-in CSS and Sass support, and support for any CSS-in-JS library
- Custom Error Handlers
- Development environment which supports Hot Module Replacement
- Cross-Platform Support

## Summary

<!-- vim-markdown-toc GFM -->

- [Installation](#installation)
- [Examples](#examples)
- [Usage](#usage)
  - [Setup](#setup)
  - [API Spec](#api-spec)
  - [Rendering Pages](#rendering-pages)
  - [Handling Errors](#handling-errors)
    - [Custom error handler](#custom-error-handler)
  - [Advanced Usage](#advanced-usage)
- [Configuration](#configuration)
- [Versioning](#versioning)
- [Contributing](#contributing)
  - [Contribution Guidelines](#contribution-guidelines)
  - [Code of Conduct](#code-of-conduct)

<!-- vim-markdown-toc -->

---

## Installation

```bash
$ yarn add nesxt
```

## Examples

See full examples [here](https://github.com/omauricf/nesxt/tree/master/examples).

## Usage

### Setup

Simply import `NesxtModule` in your Nest application.

```typescript
import { Module } from "@nestjs/common";
import { NesxtModule } from "nesxt";

@Module({
  imports: [
    NesxtModule.forRootAsync(),
    ...
  ],
  ....
})
export class AppModule {}
```

### API Spec

The `forRootAsync()` method takes an options object with some useful properties.

| Property             | Type          | Description                                                      |
| -------------------- | ------------- | ---------------------------------------------------------------- |
| `viewsDir`           | null/string   | Views directory, relative to `pages` dir. Default: `"views"`     |
| `dev`                | boolean       | Setup the development environment. Default: `"true"`             |
| `rootPath`           | string        | Static files root directory. Default: `"public"`                 |
| `serveRoot`          | string        | Root path under which static app will be served. Default: `""`   |
| `renderPath`         | string/RegExp | Path to static files (concat with the `serveRoot`). Default: `*` |
| `exclude`            | string[]      | Paths to exclude when serving static files.                      |
| `serveStaticOptions` | Object        | Serve static options (static files)                              |

### Rendering Pages

The `NesxtModule` overrides the Express/Fastify render. However, to make things easier, we introduce the `@Router` decorator, that can be imported from `nesxt` and add it to the method that will render the page. The path for the page is relative to the `/pages` directory. This is useful to keep your routes and controllers in separated files.

```typescript
import { Router, Get, Render } from "nesxt";

@Router()
export class AppRouter {
  @Get()
  @Render("Home")
  public getIndex() {
    // initial props
    return {};
  }
}
```

Additionally, the render function is made available on the `res` object.

```typescript
import { Router, Get, Res, RenderableResponse } from "nesxt";

@Router()
export class AppRouter {
  @Get("/")
  public getIndex(@Res() res: RenderableResponse) {
    return res.render("Home");
  }
}
```

The render function takes in the view, as well as the initial props passed to the page.

```typescript
render = (view: string, initialProps?: any) => any;
```

### Handling Errors

By default, errors will be handled by next's, which uses the ([customizable](https://nextjs.org/docs/#custom-error-handling)) \_error page.

#### Custom error handler

A custom error handler can be set to override or enhance it's default behavior.

**ErrorHandler Typedef**

```typescript
export type ErrorHandler = (
  err: any,
  req: any,
  res: any,
  pathname: any,
  query: ParsedUrlQuery,
) => Promise<any>;
```

**Setting ErrorHandler**

You can set the error handler by getting the NesxtModule from nest's container.

```typescript
// in main.ts file after registering the NesxtModule

const bootstrap() => {
  ...

  // get the NesxtService
  const service = server.get(NesxtService);

  service.setErrorHandler(async (err, req, res) => {
    // send JSON response
    res.send(err.response);
  });

  ...
}

```

### Advanced Usage

Passing server data to the Next.js client.

**src/app.router.ts**

```ts
@Router()
export class AppRouter {
  constructor(private readonly app: AppService) {}

  @Get("/")
  @Render("Home")
  public getIndex() {
    const { title, content } = await this.app.findAll();
    return { title, content };
  }
}
```

**pages/views/Home.tsx**

```tsx
import React from "react";
import { NextPageContext } from "next";

const Home = ({ title, content }) => {
  return (
    <div>
      <h1>{title}</h1>
      <p>{content}</p>
    </div>
  );
};

Home.getInitialProps = async ({ query }: NextPageContext) => {
  const { title, content } = query;
  return { title, content };
};

export default Home;
```

### Configuration

Update your typescript settings to support the Nesxt environment.

**tsconfig.json**

```json
{
  "compilerOptions": {
    "allowJs": true,
    "emitDecoratorMetadata": true,
    "esModuleInterop": true,
    "experimentalDecorators": true,
    "forceConsistentCasingInFileNames": true,
    "isolatedModules": true,
    "jsx": "preserve",
    "lib": ["dom", "dom.iterable", "esnext"],
    "module": "esnext",
    "moduleResolution": "node",
    "noEmit": true,
    "resolveJsonModule": true,
    "skipLibCheck": true,
    "strict": false,
    "target": "es5",
    "baseUrl": "."
  },
  "exclude": ["node_modules"],
  "include": ["next-env.d.ts", "**/*.ts", "**/*.tsx"]
}
```

**tsconfig.build.json**

```json
{
  "extends": "./tsconfig.json",
  "compilerOptions": {
    "declaration": true,
    "incremental": true,
    "module": "commonjs",
    "noEmit": false,
    "outDir": "dist",
    "removeComments": true,
    "sourceMap": true,
    "target": "es2017"
  },
  "exclude": ["node_modules", "test", "**/*spec.ts"],
  "include": ["src"]
}
```

## Versioning

The major version of `nesxt` corresponds to the major version of `next`.

## Contributing

Thank you for being interested on making this package better. We encourage everyone to help improving this project with some new features, bug fixes and performance issues. Please take a little bit of your time to read our guides, so this process can be faster and easier.

### Contribution Guidelines

Take a moment to read about our [Contribution Guidelines](/.github/CONTRIBUTING.md) so you can understand how to submit an issue, commit and create pull requests.

### Code of Conduct

We expect you to follow our [Code of Conduct](/.github/CODE_OF_CONDUCT.md). You can read it to understand what kind of behaviour will and will not be tolerated.

## Stay in touch

- Author - [Mauri Costa](https://github.com/omauricf)

## License

Nesxt is [MIT licensed](LICENSE).
